/*
 * Portions of this software are copyrighted by the 
 * 
 *     University of South Carolina,
 *     Sun Microsystems, Inc., 
 * 
 * and other parties. 
 * 
 * IN NO EVENT SHALL THE AUTHORS OR DISTRIBUTORS BE LIABLE TO ANY PARTY
 * FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES
 * ARISING OUT OF THE USE OF THIS SOFTWARE, ITS DOCUMENTATION, OR ANY
 * DERIVATIVES THEREOF, EVEN IF THE AUTHORS HAVE BEEN ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 * 
 * THE AUTHORS AND DISTRIBUTORS SPECIFICALLY DISCLAIM ANY WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE, AND NON-INFRINGEMENT.  THIS SOFTWARE
 * IS PROVIDED ON AN "AS IS" BASIS, AND THE AUTHORS AND DISTRIBUTORS HAVE
 * NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR
 * MODIFICATIONS.
 */

package edu.sc.seis.TauP;

import java.io.*;
import java.util.zip.*;

/** static class that loads a tau model, after searching for it. It can 
  * be extended to change the search mechanism. 
  *
  * @version 1.0  Tue Feb  2 20:48:34 GMT 1999
  * @author H. Philip Crotwell
  *
  */
 public class TauModelLoader {

       /** Reads the velocity model, slowness model, and tau model from
        * a file saved using Java's Serializable interface.
        */ 
 	public static TauModel load(String modelName, String searchPath) throws FileNotFoundException,
             ClassNotFoundException,
             InvalidClassException,
             IOException,
             StreamCorruptedException,
             OptionalDataException
   {
      String filename;

         /* Append ".taup" to modelname if it isn't already there. */
      if (modelName.endsWith(".taup")) {
         filename = modelName;
      } else {
         filename = modelName + ".taup";
      }

      String classPath = System.getProperty("java.class.path");
      String taupPath = searchPath;

      int offset = 0;
      int pathSepIndex;
      String pathEntry;
      File jarFile;
      File modelFile;

         /* First we try to find the model in the distributed taup.jar file.*/
      while (offset < classPath.length()) {
         pathSepIndex = classPath.indexOf(File.pathSeparatorChar, offset);
         if (pathSepIndex != -1) {
            pathEntry = classPath.substring(offset,pathSepIndex);
            offset = pathSepIndex+1;
         } else {
            pathEntry = classPath.substring(offset);
            offset = classPath.length();
         }
         jarFile = new File(pathEntry);
         if (jarFile.exists() && jarFile.isFile() &&
         jarFile.getName().equals("taup.jar") && jarFile.canRead()) {
            ZipFile zippy = new ZipFile(jarFile);
            ZipEntry zipEntry = zippy.getEntry("StdModels/"+filename);
            if (zipEntry != null) {
               return TauModel.readModelFromStream(zippy.getInputStream( zipEntry));
            }
         }
      }

         /*  It isn't in the taup.jar so we try to find it within the paths
          *  specified in the taup.model.path property. */
      offset = 0;
      if (taupPath != null) {
         while (offset < taupPath.length()) {
            pathSepIndex = taupPath.indexOf(File.pathSeparatorChar, offset);
            if (pathSepIndex != -1) {
               pathEntry = taupPath.substring(offset,pathSepIndex);
               offset = pathSepIndex+1;
            } else {
               pathEntry = taupPath.substring(offset);
               offset = taupPath.length();
            }

               /* Check each jar file. */
            if (pathEntry.endsWith(".jar") || pathEntry.endsWith(".zip")) {
               jarFile = new File(pathEntry);
               if (jarFile.exists() && jarFile.isFile() && jarFile.canRead()) {
                  ZipFile zippy = new ZipFile(jarFile);
                  ZipEntry zipEntry = zippy.getEntry("StdModels/"+filename);
                  if (zipEntry != null) {
                     return TauModel.readModelFromStream(zippy.getInputStream( zipEntry));
                  }
               }
            } else {
                  /* Check for regular files. */
               modelFile = new File(pathEntry+"/"+filename);
               if (modelFile.exists() && modelFile.isFile() &&
               modelFile.canRead()) {
                  return TauModel.readModel(modelFile.getCanonicalPath());
               }
            }
         }
      }

         /* Couldn't find it in the taup.model.path either, look in the current
          * directory. */
      modelFile = new File(filename);
      if (modelFile.exists()&& modelFile.isFile()&&modelFile.canRead()) {
         return TauModel.readModel(modelFile.getCanonicalPath());
      } else {
         throw new FileNotFoundException(
            "Can't find any saved models for "+modelName);
      }


 	}
 }
